@@ -22,21 +22,21 @@ Contains the Datastore object, which is the manager of all the active backends
 
 from collections import deque
 import threading
+import logging
 import uuid
 
 from GTG.backends.backend_signals import BackendSignals
 from GTG.backends.generic_backend import GenericBackend
 from GTG.core.config import CoreConfig
 from GTG.core import requester
-from GTG.core.dirs import PROJECTS_XMLFILE, TAGS_XMLFILE
 from GTG.core.search import parse_search_query, search_filter, InvalidQuery
-from GTG.core.tag import Tag, SEARCH_TAG
+from GTG.core.tag import Tag, SEARCH_TAG, SEARCH_TAG_PREFIX
 from GTG.core.task import Task
 from GTG.core.treefactory import TreeFactory
-from GTG.core import cleanxml
 from GTG.core.borg import Borg
-from GTG.core.logger import log, log_debug_enabled
 
+
+log = logging.getLogger(__name__)
 TAG_XMLROOT = "tagstore"
 
 
@@ -60,8 +60,9 @@ class DataStore():
         self.requester = requester.Requester(self, global_conf)
         self.tagfile_loaded = False
         self._tagstore = self.treefactory.get_tags_tree(self.requester)
-        self.load_tag_tree()
         self._backend_signals = BackendSignals()
+        self.conf = global_conf
+        self.tag_idmap = {}
 
         # Flag when turned to true, all pending operation should be
         # completed and then GTG should quit
@@ -72,7 +73,6 @@ class DataStore():
         self.is_default_backend_loaded = False
         self._backend_signals.connect('default-backend-loaded',
                                       self._activate_non_default_backends)
-        self.filtered_datastore = FilteredDataStore(self)
         self._backend_mutex = threading.Lock()
 
     # Accessor to embedded objects in DataStore ##############################
@@ -111,18 +111,18 @@ class DataStore():
         self._tagstore.add_node(tag, parent_id=parent_id)
         tag.set_save_callback(self.save)
 
-    def new_tag(self, name, attributes={}):
+    def new_tag(self, name, attributes={}, tid=None):
         """
         Create a new tag
 
         @returns GTG.core.tag.Tag: the new tag
         """
         parameters = {'tag': name}
-        tag = Tag(name, req=self.requester, attributes=attributes)
+        tag = Tag(name, req=self.requester, attributes=attributes, tid=tid)
         self._add_new_tag(name, tag, self.treefactory.tag_filter, parameters)
         return tag
 
-    def new_search_tag(self, name, query, attributes={}):
+    def new_search_tag(self, name, query, attributes={}, tid=None, save=True):
         """
         Create a new search tag
 
@@ -130,8 +130,8 @@ class DataStore():
         """
         try:
             parameters = parse_search_query(query)
-        except InvalidQuery as e:
-            log.warning(f"Problem with parsing query '{query}' (skipping): {e.message}")
+        except InvalidQuery as error:
+            log.warning("Problem with parsing query %r (skipping): %s", query, error.message)
             return None
 
         # Create own copy of attributes and add special attributes label, query
@@ -139,10 +139,14 @@ class DataStore():
         init_attr["label"] = name
         init_attr["query"] = query
 
-        tag = Tag(name, req=self.requester, attributes=init_attr)
+        name = SEARCH_TAG_PREFIX + name
+        tag = Tag(name, req=self.requester, attributes=init_attr, tid=tid)
         self._add_new_tag(name, tag, search_filter, parameters,
                           parent_id=SEARCH_TAG)
-        self.save_tagtree()
+
+        if save:
+            self.save_tagtree()
+
         return tag
 
     def remove_tag(self, name):
@@ -175,12 +179,14 @@ class DataStore():
                 # Store old tag attributes
                 color = tag.get_attribute("color")
                 icon = tag.get_attribute("icon")
+                tid = tag.tid
 
                 my_task = self.get_task(task_id)
                 my_task.rename_tag(oldname, newname)
 
                 # Restore attributes on tag
                 new_tag = self.get_tag(newname)
+                new_tag.tid = tid
 
                 if color:
                     new_tag.set_attribute("color", color)
@@ -188,6 +194,9 @@ class DataStore():
                 if icon:
                     new_tag.set_attribute("icon", icon)
 
+            self.remove_tag(oldname)
+            self.save_tagtree()
+
             return None
 
         query = tag.get_attribute("query")
@@ -198,11 +207,11 @@ class DataStore():
             newname = '_' + newname
 
         label, num = newname, 1
-        while self._tagstore.has_node(label):
+        while self._tagstore.has_node(SEARCH_TAG_PREFIX + label):
             num += 1
             label = newname + " " + str(num)
 
-        self.new_search_tag(label, query)
+        self.new_search_tag(label, query, {}, tag.tid)
 
     def get_tag(self, tagname):
         """
@@ -215,66 +224,82 @@ class DataStore():
         else:
             return None
 
-    def load_tag_tree(self):
+    def load_tag_tree(self, tag_tree):
         """
         Loads the tag tree from a xml file
         """
-        doc, xmlstore = cleanxml.openxmlfile(TAGS_XMLFILE, TAG_XMLROOT)
-        for t in xmlstore.childNodes:
-            tagname = t.getAttribute("name")
-            parent = t.getAttribute("parent")
 
-            tag_attr = {}
-            attr = t.attributes
-            for i in range(attr.length):
-                at_name = attr.item(i).name
-                if at_name not in ["name", "parent"]:
-                    at_val = t.getAttribute(at_name)
-                    tag_attr[at_name] = at_val
+        for element in tag_tree.iter('tag'):
+            tid = element.get('id')
+            name = element.get('name')
+            color = element.get('color')
+            icon = element.get('icon')
+            parent = element.get('parent')
+            nonactionable = element.get('nonactionable')
 
-            if parent == SEARCH_TAG:
-                query = t.getAttribute("query")
-                tag = self.new_search_tag(tagname, query, tag_attr)
-            else:
-                tag = self.new_tag(tagname, tag_attr)
-                if parent:
-                    tag.set_parent(parent)
+            tag_attrs = {}
+
+            if color:
+                tag_attrs['color'] = '#' + color
+
+            if icon:
+                tag_attrs['icon'] = icon
+
+            if nonactionable:
+                tag_attrs['nonactionable'] = nonactionable
+
+            tag = self.new_tag(name, tag_attrs, tid)
+
+            if parent:
+                tag.set_parent(parent)
+
+            # Add to idmap for quick lookup based on ID
+            self.tag_idmap[tid] = tag
 
         self.tagfile_loaded = True
 
+
+    def load_search_tree(self, search_tree):
+        """Load saved searches tree."""
+
+        for element in search_tree.iter('savedSearch'):
+            tid = element.get('id')
+            name = element.get('name')
+            color = element.get('color')
+            icon = element.get('icon')
+            query = element.get('query')
+
+            tag_attrs = {}
+
+            if color:
+                tag_attrs['color'] = color
+
+            if icon:
+                tag_attrs['icon'] = icon
+
+            self.new_search_tag(name, query, tag_attrs, tid, False)
+
+
+    def get_tag_by_id(self, tid):
+        """Get a tag by its ID"""
+
+        try:
+            return self.tag_idmap[tid]
+        except KeyError:
+            return
+
     def save_tagtree(self):
         """ Saves the tag tree to an XML file """
+
         if not self.tagfile_loaded:
             return
 
-        doc, xmlroot = cleanxml.emptydoc(TAG_XMLROOT)
         tags = self._tagstore.get_main_view().get_all_nodes()
-        already_saved = []
 
-        for tagname in tags:
-            if tagname in already_saved:
-                continue
+        for backend in self.backends.values():
+            if backend.get_name() == 'backend_localfile':
+                backend.save_tags(tags, self._tagstore)
 
-            tag = self._tagstore.get_node(tagname)
-            attributes = tag.get_all_attributes(butname=True, withparent=True)
-            if "special" in attributes or len(attributes) == 0:
-                continue
-
-            t_xml = doc.createElement("tag")
-            t_xml.setAttribute("name", tagname)
-            for attr in attributes:
-                # skip labels for search tags
-                if tag.is_search_tag() and attr == 'label':
-                    continue
-
-                value = tag.get_attribute(attr)
-                if value:
-                    t_xml.setAttribute(attr, value)
-
-            xmlroot.appendChild(t_xml)
-            already_saved.append(tagname)
-
-        cleanxml.savexml(TAGS_XMLFILE, doc, backup=True)
 
     # Tasks functions #########################################################
     def get_all_tasks(self):
@@ -307,7 +332,7 @@ class DataStore():
         if self.has_task(tid):
             return self._tasks.get_node(tid)
         else:
-            # log.error("requested non-existent task %s" % tid)
+            # log.error("requested non-existent task %s", tid)
             # This is not an error: it is normal to request a task which
             # might not exist yet.
             return None
@@ -403,6 +428,8 @@ class DataStore():
                 log.error("registering a backend without pid.")
                 return None
             backend = backend_dic["backend"]
+            first_run = backend_dic["first_run"]
+
             # Checking that is a new backend
             if backend.get_id() in self.backends:
                 log.error("registering already registered backend")
@@ -411,7 +438,11 @@ class DataStore():
             # filtering the tasks that should hit the backend.
             source = TaskSource(requester=self.requester,
                                 backend=backend,
-                                datastore=self.filtered_datastore)
+                                datastore=self)
+
+            if first_run:
+                backend.this_is_the_first_run(None)
+
             self.backends[backend.get_id()] = source
             # we notify that a new backend is present
             self._backend_signals.backend_added(backend.get_id())
@@ -432,7 +463,7 @@ class DataStore():
                 source.start_get_tasks()
             return source
         else:
-            log.error("Tried to register a backend without a  pid")
+            log.error("Tried to register a backend without a pid")
 
     def _activate_non_default_backends(self, sender=None):
         """
@@ -562,40 +593,50 @@ class DataStore():
 
         @param quit: If quit is true, backends are shut down
         """
+
         try:
             self.start_get_tasks_thread.join()
         except Exception:
             pass
-        doc, xmlconfig = cleanxml.emptydoc("config")
+
         # we ask all the backends to quit first.
         if quit:
             # we quit backends in parallel
             threads_dic = {}
+
             for b in self.get_all_backends():
                 thread = threading.Thread(target=b.quit)
                 threads_dic[b.get_id()] = thread
                 thread.start()
+
             for backend_id, thread in threads_dic.items():
                 # after 20 seconds, we give up
                 thread.join(20)
-                if thread.isAlive():
+
+                alive = thread.is_alive()
+
+                if alive:
                     log.error("The %s backend stalled while quitting",
                               backend_id)
+
         # we save the parameters
         for b in self.get_all_backends(disabled=True):
-            t_xml = doc.createElement("backend")
+            config = self.conf.get_backend_config(b.get_name())
+
+
             for key, value in b.get_parameters().items():
                 if key in ["backend", "xmlobject"]:
                     # We don't want parameters, backend, xmlobject:
                     # we'll create them at next startup
                     continue
+
                 param_type = b.get_parameter_type(key)
                 value = b.cast_param_type_to_string(param_type, value)
-                t_xml.setAttribute(str(key), value)
-            # Saving all the projects at close
-            xmlconfig.appendChild(t_xml)
-        cleanxml.savexml(PROJECTS_XMLFILE, doc, backup=True)
-        # Saving the tagstore
+                config.set(str(key), value)
+
+        config.save()
+
+        #  Saving the tagstore
         self.save_tagtree()
 
     def request_task_deletion(self, tid):
@@ -628,7 +669,7 @@ class TaskSource():
 
         @param requester: a Requester
         @param backend:  the backend being wrapped
-        @param datastore: a FilteredDatastore
+        @param datastore: a Datastore
         """
         self.backend = backend
         self.req = requester
@@ -638,7 +679,7 @@ class TaskSource():
         self.to_remove = deque()
         self.please_quit = False
         self.task_filter = self.get_task_filter_for_backend()
-        if log_debug_enabled():
+        if log.isEnabledFor(logging.DEBUG):
             self.timer_timestep = 5
         else:
             self.timer_timestep = 1
@@ -847,31 +888,3 @@ class TaskSource():
             return self.__dict__[attr]
         else:
             return getattr(self.backend, attr)
-
-
-class FilteredDataStore(Borg):
-    """
-    This class acts as an interface to the Datastore.
-    It is used to hide most of the methods of the Datastore.
-    The backends can safely use the remaining methods.
-    """
-
-    def __init__(self, datastore):
-        super().__init__()
-        self.datastore = datastore
-
-    def __getattr__(self, attr):
-        if attr in ['task_factory',
-                    'push_task',
-                    'get_task',
-                    'has_task',
-                    'get_all_tasks',
-                    'get_tasks_tree',
-                    'get_backend_mutex',
-                    'flush_all_tasks',
-                    'request_task_deletion']:
-            return getattr(self.datastore, attr)
-        elif attr in ['get_all_tags']:
-            return self.datastore.requester.get_all_tags
-        else:
-            raise AttributeError(f"No attribute {attr}")
