@@ -19,19 +19,21 @@
 """
 task.py contains the Task class which represents (guess what) a task
 """
-from datetime import datetime
-import cgi
+from datetime import datetime, date
+import html
 import re
 import uuid
-import xml.dom.minidom
+import logging
 import xml.sax.saxutils as saxutils
 
-from GTG.core.translations import _
+from gi.repository import GLib
+
+from gettext import gettext as _
 from GTG.core.dates import Date
-from GTG.core.logger import log
-from GTG.core.tag import extract_tags_from_text
 from liblarch import TreeNode
 
+log = logging.getLogger(__name__)
+
 
 class Task(TreeNode):
     """ This class represent a task in GTG.
@@ -41,22 +43,29 @@ class Task(TreeNode):
     STA_ACTIVE = "Active"
     STA_DISMISSED = "Dismiss"
     STA_DONE = "Done"
+    DEFAULT_TASK_NAME = None
 
     def __init__(self, task_id, requester, newtask=False):
         super().__init__(task_id)
         # the id of this task in the project should be set
         # tid is a string ! (we have to choose a type and stick to it)
-        assert(isinstance(task_id, str) or isinstance(task_id, str))
-        self.tid = str(task_id)
-        self.set_uuid(uuid.uuid4())
+        if not isinstance(task_id, str):
+            raise ValueError("Wrong type for task_id %r", type(task_id))
+        self.tid = task_id
+        self.set_uuid(task_id)
         self.remote_ids = {}
+        # set to True to disable self.sync() and avoid flooding on task edit
+        self.sync_disabled = False
         self.content = ""
-        self.title = _("My new task")
+        if Task.DEFAULT_TASK_NAME is None:
+            Task.DEFAULT_TASK_NAME = _("My new task")
+        self.title = Task.DEFAULT_TASK_NAME
         # available status are: Active - Done - Dismiss - Note
         self.status = self.STA_ACTIVE
+
         self.added_date = Date.no_date()
         if newtask:
-            self.added_date = datetime.now()
+            self.added_date = Date(datetime.now())
 
         self.closed_date = Date.no_date()
         self.due_date = Date.no_date()
@@ -74,17 +83,17 @@ class Task(TreeNode):
         self.attributes = {}
         self._modified_update()
 
+        # Setting the attributes related to repeating tasks.
+        self.recurring = False  # type: bool
+        self.recurring_term = None  # type: str
+        self.recurring_updated_date = Date.no_date()
+        self.inherit_recursion()
+
     def get_added_date(self):
         return self.added_date
 
-    def get_added_date_string(self):
-        return self.added_date.strftime("%Y-%m-%dT%H:%M:%S")
-
-    def get_added_date_simple(self):
-        return self.added_date.strftime("%Y/%m/%d") if self.added_date else ""
-
-    def set_added_date(self, date):
-        self.added_date = date
+    def set_added_date(self, value):
+        self.added_date = Date(value)
 
     def is_loaded(self):
         return self.loaded
@@ -115,98 +124,63 @@ class Task(TreeNode):
             self.sync()
         return str(self.uuid)
 
-    def get_remote_ids(self):
-        """
-        A task usually has a different id in all the different backends.
-        This function returns a dictionary backend_id->the id the task has
-        in that backend
-        @returns dict: dictionary backend_id->task remote id
-        """
-        return self.remote_ids
-
-    def add_remote_id(self, backend_id, task_remote_id):
-        """
-        A task usually has a different id in all the different backends.
-        This function adds a relationship backend_id-> remote_id that can be
-        retrieved using get_remote_ids
-        @param backend_id: string representing the backend id
-        @param task_remote_id: the id for this task in the backend backend_id
-        """
-        self.remote_ids[str(backend_id)] = str(task_remote_id)
-
     def get_title(self):
         return self.title
 
-    # Return True if the title was changed.
-    # False if the title was already the same.
+    def duplicate(self):
+        """ Duplicates a task with a new ID """
+        copy = self.req.ds.new_task()
+        # Inherit the recurrency
+        copy.set_recurring(True, self.recurring_term)
+        nextdate = self.get_next_occurrence()
+        copy.set_due_date(nextdate)
+
+        copy.set_title(self.title)
+        copy.content = self.content
+        copy.tags = self.tags
+        log.debug("Duppicating task %s as task %s",
+                  self.get_id(), copy.get_id())
+        return copy
+
+    def duplicate_recursively(self):
+        """
+        Duplicates recursively all the task itself and its children while keeping the relationship
+        """
+        newtask = self.duplicate()
+        if self.has_child():
+            for c_tid in self.get_children():
+                child = self.req.get_task(c_tid)
+                if child.is_loaded():
+                    newtask.add_child(child.duplicate_recursively())
+
+        newtask.sync()
+        return newtask.tid
+
+
     def set_title(self, title):
-        # We should check for other task with the same title
-        # In that case, we should add a number (like Tomboy does)
-        old_title = self.title
+        """Set the tasks title. Returns True if title was changed."""
+
         if title:
-            self.title = title.strip('\t\n')
+            title = title.strip('\t\n')
         else:
-            self.title = "(no title task)"
-        # Avoid unnecessary sync
-        if self.title != old_title:
+            title = '(no title task)'
+
+        # Avoid unnecessary syncing
+        if title == self.title:
+            return False
+        else:
+            self.title = title
             self.sync()
             return True
+
+    def toggle_status(self):
+
+        if self.status in [self.STA_DONE, self.STA_DISMISSED]:
+            self.set_status(self.STA_ACTIVE)
         else:
-            return False
+            self.set_status(self.STA_DONE)
 
-    # TODO: should we merge this function with set_title ?
-    def set_complex_title(self, text, tags=[]):
-        if tags:
-            assert(isinstance(tags[0], str))
-        due_date = Date.no_date()
-        defer_date = Date.no_date()
-        if text:
-            # Get tags in the title
-            for match in extract_tags_from_text(text):
-                tags.append(match)
-            # Get attributes
-            regexp = r'([\s]*)([\w-]+):\s*([^\s]+)'
-            matches = re.findall(regexp, text, re.UNICODE)
-            for spaces, attribute, args in matches:
-                valid_attribute = True
-                if attribute.lower() in ["tags", _("tags"), "tag", _("tag")]:
-                    for tag in args.split(","):
-                        if not tag.strip() == "@" and not tag.strip() == "":
-                            if not tag.startswith("@"):
-                                tag = "@" + tag
-                            tags.append(tag)
-                elif attribute.lower() in ["defer", _("defer"), "start",
-                                           _("start")]:
-                    try:
-                        defer_date = Date.parse(args)
-                    except ValueError:
-                        valid_attribute = False
-                elif attribute.lower() == "due" or \
-                        attribute.lower() == _("due"):
-                    try:
-                        due_date = Date.parse(args)
-                    except:
-                        valid_attribute = False
-                else:
-                    # attribute is unknown
-                    valid_attribute = False
-
-                if valid_attribute:
-                    # remove valid attribute from the task title
-                    text = \
-                        text.replace(f"{spaces}{attribute}:{args}", "")
-
-            for t in tags:
-                self.add_tag(t)
-
-            if text != "":
-                self.set_title(text.strip())
-                self.set_to_keep()
-
-            self.set_due_date(due_date)
-            self.set_start_date(defer_date)
-
-    def set_status(self, status, donedate=None):
+    def set_status(self, status, donedate=None, propagation=False, init=False):
         old_status = self.status
         self.can_be_deleted = False
         # No need to update children or whatever if the task is not loaded
@@ -216,7 +190,26 @@ class Task(TreeNode):
             if status in [self.STA_DONE, self.STA_DISMISSED]:
                 for c in self.get_subtasks():
                     if c.get_status() in [self.STA_ACTIVE]:
-                        c.set_status(status, donedate=donedate)
+                        c.set_status(status, donedate=donedate, propagation=True)
+
+                # If the task is recurring, it must be duplicate with
+                # another task id and the next occurence of the task
+                # while preserving child/parent relations.
+                # For a task to be duplicated, it must satisfy 3 rules.
+                #   1- It is recurring.
+                #   2- It has no parent or no recurring parent.
+                #   3- It was directly marked as done (not by propagation from its parent).
+                rules = [self.recurring, not propagation]
+                if all(rules) and not self.is_parent_recurring():
+                    # duplicate all the children
+                    nexttask_tid = self.duplicate_recursively()
+                    if self.has_parent():
+                        for p_tid in self.get_parents():
+                            par = self.req.get_task(p_tid)
+                            if par.is_loaded() and par.get_status() in (self.STA_ACTIVE):
+                                par.add_child(nexttask_tid)
+                                par.sync()
+
             # If we mark a task as Active and that some parent are not
             # Active, we break the parent/child relation
             # It has no sense to have an active subtask of a done parent.
@@ -237,6 +230,8 @@ class Task(TreeNode):
 
         # then the task itself
         if status:
+            if not init:
+                GLib.idle_add(self.req.emit, "status-changed", self.tid, self.status, status)
             self.status = status
 
         # Set closing date
@@ -255,11 +250,8 @@ class Task(TreeNode):
     def get_modified(self):
         return self.last_modified
 
-    def get_modified_string(self):
-        return self.last_modified.strftime("%Y-%m-%dT%H:%M:%S")
-
-    def set_modified(self, modified):
-        self.last_modified = modified
+    def set_modified(self, value):
+        self.last_modified = Date(value)
 
     def recursive_sync(self):
         """Recursively sync the task and all task children. Defined"""
@@ -268,6 +260,166 @@ class Task(TreeNode):
             sub = self.req.get_task(sub_id)
             sub.recursive_sync()
 
+    # ABOUT RECURRING TASKS
+    # Like anything related to dates, repeating tasks are subtle and complex
+    # when creating a new task, the due date is calculated from either the current date or
+    # the start date, while we get the next occurrence of a task not from the current date but
+    # from the due date itself.
+    #
+    # However when we are retrieving the task from the XML files, we should only set the
+    # the recurring_term.
+
+    def set_recurring(self, recurring: bool, recurring_term: str = None, newtask=False):
+        """Sets a task as recurring or not, and its recurring term.
+
+        There are 4 cases to acknowledge when setting a task to recurring:
+          - if repeating but the term is invalid: it will be set to False.
+          - if repeating and the term is valid: we set it to True.
+          - if not repeating and the term is valid: we set the bool attr to True and set the term.
+          - if not repeating and the term is invalid: we set it to False and keep the previous term.
+
+        Setting a task as recurrent implies that the
+        children of a recurrent task will be also
+        set to recurrent and will inherit
+        their parent's recurring term
+
+        Args:
+            recurring (bool): True if the task is recurring and False if not.
+            recurring_term (str, optional): the recurring period of a task (every Monday, day..).
+                                            Defaults to None.
+            newtask (bool, optional): if this is a new task, we must set the due_date.
+                                      Defaults to False.
+        """
+        def is_valid_term():
+            """ Verify if the term is valid and returns the appropriate Due date.
+
+            Return a tuple of (bool, Date)
+            """
+            if recurring_term is None:
+                return False, None
+
+            try:
+                # If a start date is already set,
+                # we should calculate the next date from that day.
+                if self.start_date == Date.no_date():
+                    start_from = Date(datetime.now())
+                else:
+                    start_from = self.start_date
+
+                newdate = start_from.parse_from_date(recurring_term, newtask)
+                return True, newdate
+            except ValueError:
+                return False, None
+
+        self.recurring = recurring
+        # We verifiy if the term passed is valid
+        valid, newdate = is_valid_term()
+
+        recurring_term = recurring_term if valid else None
+
+        if self.recurring:
+            if not valid:
+                self.recurring_term = None
+                self.recurring = False
+            else:
+                self.recurring_term = recurring_term
+                self.recurring_updated_date = datetime.now()
+                if newtask:
+                    self.set_due_date(newdate)
+        else:
+            if valid:
+                self.recurring_term = recurring_term
+                self.recurring_updated_date = datetime.now()
+
+        self.sync()
+        # setting its children to recurrent
+        if self.has_child():
+            for c_tid in self.get_children():
+                child = self.req.get_task(c_tid)
+                if child.is_loaded() and child.get_status() in (self.STA_ACTIVE):
+                    child.set_recurring(self.recurring, self.recurring_term)
+                    if self.recurring:
+                        child.set_due_date(newdate)
+
+    def toggle_recurring(self):
+        """ Toggle a task's recurrency ON/OFF. Use this function to toggle, not set_recurring"""
+        # If there is no recurring_term, We assume it to recur every day.
+        newtask = False
+        if self.recurring_term is None:
+            self.recurring_term = 'day'
+            newtask = True
+
+        self.set_recurring(not self.recurring, self.recurring_term, newtask)
+
+    def get_recurring(self):
+        return self.recurring
+
+    def get_recurring_term(self):
+        return self.recurring_term
+
+    def get_recurring_updated_date(self):
+        return self.recurring_updated_date
+
+    def set_recurring_updated_date(self, date):
+        self.recurring_updated_date = Date(date)
+
+    def inherit_recursion(self):
+        """ Inherits the recurrent state of the parent.
+                If the task has a recurrent parent, it must be set to recur, itself.
+        """
+        if self.has_parent():
+            for p_tid in self.get_parents():
+                par = self.req.get_task(p_tid)
+                if par.get_recurring() and par.is_loaded():
+                    self.set_recurring(True, par.get_recurring_term())
+                    self.set_due_date(par.due_date)
+        else:
+            self.set_recurring(False)
+
+
+    def get_next_occurrence(self):
+        """Calcutate the next occurrence of a recurring task
+
+        To know which is the correct next occurrence there are two rules:
+        - if the task was marked as done before or during the open period (before the duedate);
+          in this case, we need to deal with the issue of tasks that recur on the same date.
+          example: due_date is 09/09 and done_date is 09/09
+        - if the task was marked after the due date, we need to figure out the next occurrence
+          after the current date(today).
+
+        Raises:
+            ValueError: if the recurring_term is invalid
+
+        Returns:
+            Date: the next due date of a task
+        """
+        today = date.today()
+        if today <= self.due_date:
+            try:
+                nextdate = self.due_date.parse_from_date(self.recurring_term, newtask=False)
+                while nextdate <= self.due_date:
+                    nextdate = nextdate.parse_from_date(self.recurring_term, newtask=False)
+                return nextdate
+            except Exception:
+                raise ValueError(f'Invalid recurring term {self.recurring_term}')
+        elif today > self.due_date:
+            try:
+                next_date = self.due_date.parse_from_date(self.recurring_term, newtask=False)
+                while next_date < date.today():
+                    next_date = next_date.parse_from_date(self.recurring_term, newtask=False)
+                return next_date
+            except Exception:
+                raise ValueError(f'Invalid recurring term {self.recurring_term}')
+
+    def is_parent_recurring(self):
+        if self.has_parent():
+            for p_tid in self.get_parents():
+                p = self.req.get_task(p_tid)
+                if p.is_loaded() and p.get_status() in (self.STA_ACTIVE) and p.get_recurring():
+                    return True
+        return False
+
+
     # ABOUT DUE DATES
     #
     # PLEASE READ THIS: although simple in appearance, handling task dates can
@@ -338,11 +490,6 @@ class Task(TreeNode):
         self.due_date = new_duedate_obj
         # If the new date is fuzzy or undefined, we don't update related tasks
         if not new_duedate_obj.is_fuzzy():
-            # if the task's start date happens later than the
-            # new due date, we update it (except for fuzzy dates)
-            if not self.get_start_date().is_fuzzy() and \
-                    self.get_start_date() > new_duedate_obj:
-                self.set_start_date(new_duedate)
             # if some ancestors' due dates happen before the task's new
             # due date, we update them (except for fuzzy dates)
             for par in __get_defined_parent_list(self):
@@ -416,20 +563,8 @@ class Task(TreeNode):
     #
     # Start date is the date at which the user has decided to work or consider
     # working on this task.
-    #
-    # The only constraint applied to start dates is that start dates cannot
-    # happen later than the task due date.
-    #
-    # The task due date (and any constrained relatives) is updated if a new
-    # task start date is chosen that does not respect this rule.
-    #
-    # Undefined/fizzy start dates don't constraint the task due date.
     def set_start_date(self, fulldate):
         self.start_date = Date(fulldate)
-        if not Date(fulldate).is_fuzzy() and \
-            not self.due_date.is_fuzzy() and \
-                Date(fulldate) > self.due_date:
-            self.set_due_date(fulldate)
         self.sync()
 
     def get_start_date(self):
@@ -481,20 +616,25 @@ class Task(TreeNode):
         # defensive programmation to avoid returning None
         if self.content:
             txt = self.content
+
+            # Prevent issues with & in content
+            txt = saxutils.escape(txt)
+            txt = txt.strip()
+
             if strip_tags:
                 for tag in self.get_tags_name():
-                    txt = self._strip_tag(txt, tag)
-            element = xml.dom.minidom.parseString(txt)
-            txt = self.__strip_content(element, strip_subtasks=strip_subtasks)
-            txt = txt.strip()
-            # We keep the desired number of lines
-            if lines > 0:
-                liste = txt.splitlines()
-                for i in liste:
-                    if i.strip() == "":
-                        liste.remove(i)
-                to_keep = liste[:lines]
-                txt = '\n'.join(to_keep)
+                    txt = (txt.replace(f'@{tag}, ', '')
+                              .replace(f'@{tag},', '')
+                              .replace(f'@{tag}', ''))
+
+            if strip_subtasks:
+                txt = re.sub(r'\{\!.+\!\}', '', txt)
+
+            # Strip blank lines and get desired amount of lines
+            txt = [line for line in txt.splitlines() if line]
+            txt = txt[:lines]
+            txt = '\n'.join(txt)
+
             # We keep the desired number of char
             if char > 0:
                 txt = txt[:char]
@@ -518,16 +658,7 @@ class Task(TreeNode):
 
     def set_text(self, texte):
         self.can_be_deleted = False
-        if texte != "<content/>":
-            # defensive programmation to filter bad formatted tasks
-            if not texte.startswith("<content>"):
-                texte = cgi.escape(texte, quote=True)
-                texte = f"<content>{texte}"
-            if not texte.endswith("</content>"):
-                texte = f"{texte}</content>"
-            self.content = str(texte)
-        else:
-            self.content = ''
+        self.content = html.unescape(str(texte))
 
     # SUBTASKS ###############################################################
     def new_subtask(self):
@@ -544,17 +675,22 @@ class Task(TreeNode):
 
         @param child: the added task
         """
-        log.debug(f"adding child {tid} to task {self.get_id()}")
+        log.debug("adding child %s to task %s", tid, self.get_id())
         self.can_be_deleted = False
         # the core of the method is in the TreeNode object
         TreeNode.add_child(self, tid)
         # now we set inherited attributes only if it's a new task
         child = self.req.get_task(tid)
         if self.is_loaded() and child and child.can_be_deleted:
-            child.set_start_date(self.get_start_date())
-            child.set_due_date(self.get_due_date())
+            # If the the child is repeating no need to change the date
+            if not child.get_recurring():
+                child.set_start_date(self.get_start_date())
+                child.set_due_date(self.get_due_date())
             for t in self.get_tags():
                 child.add_tag(t.get_name())
+
+            child.inherit_recursion()
+
         self.sync()
         return True
 
@@ -577,19 +713,6 @@ class Task(TreeNode):
         tree = self.get_tree()
         return [tree.get_node(node_id) for node_id in self.get_children()]
 
-    # FIXME: why is this function used ? It's higly specific. Remove it?
-    #        (Lionel)
-    # Agreed. it's only used by the "add tag to all subtasks" widget.
-    def get_self_and_all_subtasks(self, active_only=False, tasks=[]):
-        print("DEPRECATED FUNCTION: get_self_and_all_subtasks")
-        tasks.append(self)
-        for tid in self.get_children():
-            i = self.req.get_task(tid)
-            if i:
-                if not active_only or i.status == self.STA_ACTIVE:
-                    i.get_self_and_all_subtasks(active_only, tasks)
-        return tasks
-
     def set_parent(self, parent_id):
         """Update the task's parent. Refresh due date constraints."""
         TreeNode.set_parent(self, parent_id)
@@ -600,6 +723,7 @@ class Task(TreeNode):
                 not self.due_date.is_fuzzy() and \
                     par_duedate < self.due_date:
                 self.set_due_date(par_duedate)
+            self.inherit_recursion()
         self.recursive_sync()
 
     def set_attribute(self, att_name, att_value, namespace=""):
@@ -621,13 +745,14 @@ class Task(TreeNode):
         return self.attributes.get((namespace, att_name), None)
 
     def sync(self):
+        if self.sync_disabled:
+            return self.is_loaded()
         self._modified_update()
         if self.is_loaded():
             # This is a liblarch call to the TreeNode ancestor
             self.modified()
             return True
-        else:
-            return False
+        return False
 
     def _modified_update(self):
         """
@@ -642,13 +767,13 @@ class Task(TreeNode):
 
     # return a copy of the list of tag objects
     def get_tags(self):
-        l = []
+        tags = []
         for tname in self.tags:
             tag = self.req.get_tag(tname)
             if not tag:
                 tag = self.req.new_tag(tname)
-            l.append(tag)
-        return l
+            tags.append(tag)
+        return tags
 
     def rename_tag(self, old, new):
         eold = saxutils.escape(saxutils.unescape(old))
@@ -661,17 +786,24 @@ class Task(TreeNode):
         self.req.get_tag(new).modified()
         self.sync()
 
+    def tag_added_by_id(self, tid):
+        """Add a tag by its ID"""
+
+        tag = self.req.ds.get_tag_by_id(tid)
+
+        if tag:
+            self.tag_added(tag.get_name())
+
     def tag_added(self, tagname):
         """
         Adds a tag. Does not add '@tag' to the contents. See add_tag
         """
-        # Do not add the same tag twice
         if tagname not in self.tags:
             self.tags.append(tagname)
             if self.is_loaded():
                 for child in self.get_subtasks():
                     if child.can_be_deleted:
-                        child.add_tag(tagname)
+                        child.tag_added(tagname)
 
                 tag = self.req.get_tag(tagname)
                 if not tag:
@@ -681,29 +813,26 @@ class Task(TreeNode):
 
     def add_tag(self, tagname):
         "Add a tag to the task and insert '@tag' into the task's content"
+
         if self.tag_added(tagname):
             c = self.content
-
-            # strip <content>...</content> tags
-            if c.startswith('<content>'):
-                c = c[len('<content>'):]
-            if c.endswith('</content>'):
-                c = c[:-len('</content>')]
+            tagname = html.escape(tagname)
+            tagname = '@' + tagname if not tagname.startswith('@') else tagname
 
             if not c:
                 # don't need a separator if it's the only text
                 sep = ''
-            elif c.startswith('<tag>'):
+            elif c.startswith('@'):
                 # if content starts with a tag, make a comma-separated list
                 sep = ', '
             else:
                 # other text at the beginning, so put the tag on its own line
                 sep = '\n\n'
 
-            self.content = "<content><tag>%s</tag>%s%s</content>" % (
-                cgi.escape(tagname), sep, c)
+            self.content = f'{tagname}{sep}{c}'
             # we modify the task internal state, thus we have to call for a
             # sync
+
             self.sync()
 
     # remove by tagname
@@ -720,32 +849,21 @@ class Task(TreeNode):
             tag = self.req.get_tag(tagname)
             # The ViewCount of the tag still doesn't know that
             # the task was removed. We need to update manually
-            tag.update_task(self.get_id())
             if tag:
+                tag.update_task(self.get_id())
                 tag.modified()
 
-    def set_only_these_tags(self, tags_list):
-        """
-        Given a list of strings representing tags, it makes sure that
-        this task has those and only those tags.
-        """
-        for tag in self.get_tags_name():
-            try:
-                tags_list.remove(tag)
-            except:
-                self.remove_tag(tag)
-        for tag in tags_list:
-            self.add_tag(tag)
-
     def _strip_tag(self, text, tagname, newtag=''):
-        inline_tag = tagname[1:]
+        if tagname.startswith('@'):
+            inline_tag = tagname[1:]
+        else:
+            inline_tag = tagname
+
         return (text
-                .replace(f'<tag>{tagname}</tag>\n\n', newtag)  # trail \n
-                # trail comma
-                .replace(f'<tag>{tagname}</tag>, ', newtag)
-                .replace(f'<tag>{tagname}</tag>,', newtag)
-                .replace(f'<tag>{tagname}</tag>', newtag)
-                # in case XML is missing (bug #504899)
+                .replace(f'@{tagname}\n\n', newtag)
+                .replace(f'@{tagname}\n', newtag)
+                .replace(f'@{tagname}, ', newtag)
+                .replace(f'@{tagname}', newtag)
                 .replace(f'{tagname}\n\n', newtag)
                 .replace(f'{tagname}, ', newtag)
                 .replace(f'{tagname},', inline_tag)
@@ -762,9 +880,10 @@ class Task(TreeNode):
                 toreturn = True
             else:
                 tag = self.req.get_tag(tagname)
-                for tagc_name in tag.get_children():
-                    if not toreturn:
-                        toreturn = children_tag(tagc_name)
+                if tag:
+                    for tagc_name in tag.get_children():
+                        if not toreturn:
+                            toreturn = children_tag(tagc_name)
             return toreturn
 
         # We want to see if the task has no tags
@@ -785,9 +904,30 @@ class Task(TreeNode):
         return toreturn
 
     def __str__(self):
-        return '<Task title="%s" id="%s" status="%s" tags="%s" added="%s">' % (
-                self.title,
-                self.tid,
-                self.status,
-                str(self.tags),
-                str(self.added_date))
+        return '<Task title="%s" id="%s" status="%s" tags="%s" added="%s" recurring="%s">' % (
+            self.title,
+            self.tid,
+            self.status,
+            self.tags,
+            self.added_date,
+            self.recurring)
+
+    __repr__ = __str__
+
+
+class DisabledSyncCtx:
+    """Context manager disabling GTG.core.task.Task.sync()
+    and firing one only sync on __exit__"""
+
+    def __init__(self, task: Task, sync_on_exit: bool = True):
+        self.task = task
+        self.sync_on_exit = sync_on_exit
+
+    def __enter__(self):
+        self.task.sync_disabled = True
+        return self.task
+
+    def __exit__(self, *args, **kwargs):
+        self.task.sync_disabled = False
+        if self.sync_on_exit:
+            self.task.sync()
