@@ -25,7 +25,7 @@ from gi.repository import GObject, Gtk, Pango
 from GTG.core.search import parse_search_query, search_filter
 from GTG.core.tag import SEARCH_TAG
 from GTG.core.task import Task
-from GTG.core.translations import _
+from gettext import gettext as _
 from GTG.gtk import colors
 from GTG.gtk.browser.cell_renderer_tags import CellRendererTags
 from GTG.core.dates import Date
@@ -65,14 +65,14 @@ class TreeviewFactory():
                     real_count = real_count + 1
         return display_count < real_count
 
-    def task_bg_color(self, node, default_color):
+    def get_task_bg_color(self, node, default_color):
         if self.config.get('bg_color_enable'):
-            return colors.background_color(node.get_tags(), default_color)
+            return colors.background_color(node.get_tags(), default_color, 0.5)
         else:
             return None
 
-    # return an ordered list of tags of a task
-    def task_tags_column(self, node):
+    def get_task_tags_column_contents(self, node):
+        """Returns an ordered list of tags of a task"""
         tags = node.get_tags()
 
         search_parent = self.req.get_tag(SEARCH_TAG)
@@ -88,26 +88,28 @@ class TreeviewFactory():
         tags.sort(key=lambda x: x.get_name())
         return tags
 
-    # task title
-    def task_title_column(self, node):
+    def get_task_title_column_string(self, node):
         return saxutils.escape(node.get_title())
 
-    # task title/label
-    def task_label_column(self, node):
+    def get_task_label_column_string(self, node):
         str_format = "%s"
 
+        # We add the indicator when task is repeating
+        INDICATOR = "\u21BB "
+        if node.get_recurring():
+            str_format = INDICATOR + str_format
+
         if node.get_status() == Task.STA_ACTIVE:
             # we mark in bold tasks which are due today or as Now
             days_left = node.get_days_left()
             if days_left is not None and days_left <= 0:
-                str_format = "<b>%s</b>"
+                str_format = f"<b>{str_format}</b>"
             if self._has_hidden_subtask(node):
                 str_format = f"<span color='{self.unactive_color}'>{str_format}</span>"
 
         title = str_format % saxutils.escape(node.get_title())
         if node.get_status() == Task.STA_ACTIVE:
-            count = self.mainview.node_n_children(node.get_id(),
-                                                  recursive=True)
+            count = self.mainview.node_n_children(node.get_id(), recursive=True)
             if count != 0:
                 title += f" ({count})"
         elif node.get_status() == Task.STA_DISMISSED:
@@ -121,120 +123,103 @@ class TreeviewFactory():
                 % (self.unactive_color, excerpt)
         return title
 
-    # task start date
-    def task_sdate_column(self, node):
-        return node.get_start_date().to_readable_string()
+    def get_task_startdate_column_string(self, node):
+        start_date = node.get_start_date()
+        if start_date:
+            return _(start_date.to_readable_string())
+        else:
+            # Do not parse with gettext then, or you'll get undefined behavior.
+            return ""
 
-    def task_adate_column(self, node):
-        return node.get_added_date_simple()
-
-    def task_duedate_column(self, node):
-        # We show the most constraining due date for task with no due dates.
+    def get_task_duedate_column_string(self, node):
+        # For tasks with no due dates, we use the most constraining due date.
         if node.get_due_date() == Date.no_date():
+            # This particular call must NOT use the gettext "_" function,
+            # as you will get some very weird erratic behavior:
+            # strings showing up and changing on the fly when the mouse hovers,
+            # whereas no strings should even be shown at all.
             return node.get_due_date_constraint().to_readable_string()
         else:
             # Other tasks show their due date (which *can* be fuzzy)
-            return node.get_due_date().to_readable_string()
+            return _(node.get_due_date().to_readable_string())
 
-    def task_cdate_column(self, node):
-        return node.get_closed_date().to_readable_string()
+    def get_task_closeddate_column_string(self, node):
+        closed_date = node.get_closed_date()
+        if closed_date:
+            return _(closed_date.to_readable_string())
+        else:
+            # Do not parse with gettext then, or you'll get undefined behavior.
+            return ""
 
-    def start_date_sorting(self, task1, task2, order):
-        sort = self.__date_comp(task1, task2, 'start', order)
-        return sort
+    def sort_by_startdate(self, task1, task2, order):
+        t1 = task1.get_start_date()
+        t2 = task2.get_start_date()
+        return self.__date_comp_continue(task1, task2, order, t1, t2)
 
-    def added_date_sorting(self, task1, task2, order):
-        sort = self.__date_comp(task1, task2, 'added', order)
-        return sort
+    def sort_by_duedate(self, task1, task2, order):
+        t1 = task1.get_urgent_date()
+        t2 = task2.get_urgent_date()
+        if t1 == Date.no_date():
+            t1 = task1.get_due_date_constraint()
+        if t2 == Date.no_date():
+            t2 = task2.get_due_date_constraint()
+        return self.__date_comp_continue(task1, task2, order, t1, t2)
 
-    def due_date_sorting(self, task1, task2, order):
-        sort = self.__date_comp(task1, task2, 'due', order)
-        return sort
+    def sort_by_closeddate(self, task1, task2, order):
+        t1 = task1.get_closed_date()
+        t2 = task2.get_closed_date()
 
-    def closed_date_sorting(self, task1, task2, order):
-        sort = self.__date_comp(task1, task2, 'closed', order)
-        return sort
+        # Convert both times to datetimes (accurate comparison)
+        if isinstance(t1, Date):
+            d = t1.date()
+            t1 = datetime(year=d.year, month=d.month, day=d.day)
+        if isinstance(t2, Date):
+            d = t2.date()
+            t2 = datetime(year=d.year, month=d.month, day=d.day)
+        return self.__date_comp_continue(task1, task2, order, t1, t2)
 
-    def title_sorting(self, task1, task2, order):
-        t1 = task1.get_title()
-        t2 = task2.get_title()
+    def sort_by_title(self, task1, task2, order):
+        # Strip "@" and convert everything to lowercase to allow fair comparisons;
+        # otherwise, Capitalized Tasks get sorted after their lowercase equivalents,
+        # and tasks starting with a tag would get sorted before everything else.
+        t1 = task1.get_title().replace("@", "").lower()
+        t2 = task2.get_title().replace("@", "").lower()
         return (t1 > t2) - (t1 < t2)
 
-    def __date_comp(self, task1, task2, para, order):
-        """This is a quite complex method to sort tasks by date,
-        handling fuzzy date and complex situation.
-        Return -1 if nid1 is before nid2, return 1 otherwise
-        """
-        if task1 and task2:
-            if para == 'start':
-                t1 = task1.get_start_date()
-                t2 = task2.get_start_date()
-            elif para == 'due':
-                t1 = task1.get_urgent_date()
-                t2 = task2.get_urgent_date()
-                if t1 == Date.no_date():
-                    t1 = task1.get_due_date_constraint()
-                if t2 == Date.no_date():
-                    t2 = task2.get_due_date_constraint()
-            elif para == 'closed':
-                t1 = task1.get_closed_date()
-                t2 = task2.get_closed_date()
-            elif para == 'added':
-                t1 = task1.get_added_date()
-                t2 = task2.get_added_date()
+    def __date_comp_continue(self, task1, task2, order, t1, t2):
+        sort = (t2 > t1) - (t2 < t1)
 
-                # Convert both times to datetimes (accurate comparison)
-                if isinstance(t1, Date):
-                    d = t1.date()
-                    t1 = datetime(year=d.year, month=d.month, day=d.day)
-                if isinstance(t2, Date):
-                    d = t2.date()
-                    t2 = datetime(year=d.year, month=d.month, day=d.day)
-            else:
-                raise ValueError(
-                    'invalid date comparison parameter: %s') % para
-            sort = (t2 > t1) - (t2 < t1)
-        else:
-            sort = 0
+        if sort != 0:  # Ingore order, since this will be done automatically
 
-        # local function
-        def reverse_if_descending(s):
-            """Make a cmpare result relative to the top instead of following
-               user-specified sort direction"""
-            if order == Gtk.SortType.ASCENDING:
-                return s
-            else:
-                return -1 * s
+            return sort
 
-        if sort == 0:
-            # Group tasks with the same tag together for visual cleanness
-            t1_tags = task1.get_tags_name()
-            t1_tags.sort()
-            t2_tags = task2.get_tags_name()
-            t2_tags.sort()
-            cmp_tags = (t1_tags > t2_tags) - (t1_tags < t2_tags)
-            sort = reverse_if_descending(cmp_tags)
+        # Dates are equal
+        # Group tasks with the same tag together for visual cleanness
+        t1_tags = task1.get_tags_name()
+        t1_tags.sort()
+        t2_tags = task2.get_tags_name()
+        t2_tags.sort()
+        sort = (t1_tags > t2_tags) - (t1_tags < t2_tags)
 
-        if sort == 0:
+        if sort == 0:  # Even tags are equal
             # Break ties by sorting by title
-            t1_title = task1.get_title()
-            t2_title = task2.get_title()
-            t1_title = locale.strxfrm(t1_title)
-            t2_title = locale.strxfrm(t2_title)
-            cmp_title = (t1_title > t2_title) - (t1_title < t2_title)
-            sort = reverse_if_descending(cmp_title)
+            sort = locale.strcoll(task1.get_title(), task2.get_title())
 
+        if order != Gtk.SortType.ASCENDING:
+            return -sort
         return sort
 
     #############################
     # Functions for tags columns
     #############################
-    def tag_name(self, node):
+    def get_tag_name(self, node):
         label = node.get_attribute("label")
         if label.startswith('@'):
             label = label[1:]
 
-        if node.get_attribute("nonworkview") == "True":
+        if node.get_attribute("nonactionable") == "True":
+            return f"<span color='{self.unactive_color}'>{label}</span>"
+        elif node.get_id() == 'search' and not node.get_children():
             return f"<span color='{self.unactive_color}'>{label}</span>"
         else:
             return label
@@ -265,7 +250,7 @@ class TreeviewFactory():
             t2_order = t2.get_attribute("order")
             return (t1_order > t2_order) - (t1_order < t2_order)
 
-    def ontag_task_dnd(self, source, target):
+    def on_tag_task_dnd(self, source, target):
         task = self.req.get_task(source)
         if target.startswith('@'):
             task.add_tag(target)
@@ -293,7 +278,7 @@ class TreeviewFactory():
         # Tags color
         col_name = 'color'
         col = {}
-        render_tags = CellRendererTags()
+        render_tags = CellRendererTags(self.config)
         render_tags.set_property('ypad', 5)
         col['title'] = _("Tags")
         col['renderer'] = ['tag', render_tags]
@@ -309,7 +294,7 @@ class TreeviewFactory():
         render_text = Gtk.CellRendererText()
         render_text.set_property('ypad', 5)
         col['renderer'] = ['markup', render_text]
-        col['value'] = [str, self.tag_name]
+        col['value'] = [str, self.get_tag_name]
         col['expandable'] = True
         col['new_column'] = False
         col['order'] = 2
@@ -333,39 +318,31 @@ class TreeviewFactory():
 
     def active_tasks_treeview(self, tree):
         # Build the title/label/tags columns
-        desc = self.common_desc_for_tasks(tree, "Tasks")
-
-        # "startdate" column
-        col_name = 'Added'
-        col = {}
-        col['title'] = _("Added date")
-        col['expandable'] = False
-        col['resizable'] = True
-        col['value'] = [str, self.task_adate_column]
-        col['order'] = 3
-        col['sorting_func'] = self.added_date_sorting
-        desc[col_name] = col
+        # Translators: Column name, containing the task titles
+        desc = self.common_desc_for_tasks(tree, _("Tasks"))
 
         # "startdate" column
         col_name = 'startdate'
         col = {}
-        col['title'] = _("Start date")
+        # Translators: Column name, containing the start date
+        col['title'] = _("Start Date")
         col['expandable'] = False
         col['resizable'] = False
-        col['value'] = [str, self.task_sdate_column]
+        col['value'] = [str, self.get_task_startdate_column_string]
         col['order'] = 3
-        col['sorting_func'] = self.start_date_sorting
+        col['sorting_func'] = self.sort_by_startdate
         desc[col_name] = col
 
         # 'duedate' column
         col_name = 'duedate'
         col = {}
+        # Translators: Column name, containing the due date
         col['title'] = _("Due")
         col['expandable'] = False
         col['resizable'] = False
-        col['value'] = [str, self.task_duedate_column]
+        col['value'] = [str, self.get_task_duedate_column_string]
         col['order'] = 4
-        col['sorting_func'] = self.due_date_sorting
+        col['sorting_func'] = self.sort_by_duedate
         desc[col_name] = col
 
         # Returning the treeview
@@ -375,17 +352,19 @@ class TreeviewFactory():
 
     def closed_tasks_treeview(self, tree):
         # Build the title/label/tags columns
-        desc = self.common_desc_for_tasks(tree, "Closed Tasks")
+        # Translators: Column name, containing the task titles
+        desc = self.common_desc_for_tasks(tree, _("Closed Tasks"))
 
         # "startdate" column
         col_name = 'closeddate'
         col = {}
-        col['title'] = _("Closed date")
+        # Translators: Column name, containing the closed date
+        col['title'] = _("Closed Date")
         col['expandable'] = False
         col['resizable'] = False
-        col['value'] = [str, self.task_cdate_column]
+        col['value'] = [str, self.get_task_closeddate_column_string]
         col['order'] = 3
-        col['sorting_func'] = self.closed_date_sorting
+        col['sorting_func'] = self.sort_by_closeddate
         desc[col_name] = col
 
         # Returning the treeview
@@ -420,19 +399,19 @@ class TreeviewFactory():
         render_text = Gtk.CellRendererText()
         render_text.set_property("ellipsize", Pango.EllipsizeMode.END)
         col['renderer'] = ['markup', render_text]
-        col['value'] = [str, self.task_title_column]
+        col['value'] = [str, self.get_task_title_column_string]
         col['visible'] = False
         col['order'] = 0
-        col['sorting_func'] = self.title_sorting
+        col['sorting_func'] = self.sort_by_title
         desc[col_name] = col
 
         # "tags" column (no title)
         col_name = 'tags'
         col = {}
-        render_tags = CellRendererTags()
+        render_tags = CellRendererTags(self.config)
         render_tags.set_property('xalign', 0.0)
         col['renderer'] = ['tag_list', render_tags]
-        col['value'] = [GObject.TYPE_PYOBJECT, self.task_tags_column]
+        col['value'] = [GObject.TYPE_PYOBJECT, self.get_task_tags_column_contents]
         col['expandable'] = False
         col['resizable'] = False
         col['order'] = 1
@@ -441,11 +420,11 @@ class TreeviewFactory():
         # "label" column
         col_name = 'label'
         col = {}
-        col['title'] = _(title_label)
+        col['title'] = title_label
         render_text = Gtk.CellRendererText()
         render_text.set_property("ellipsize", Pango.EllipsizeMode.END)
         col['renderer'] = ['markup', render_text]
-        col['value'] = [str, self.task_label_column]
+        col['value'] = [str, self.get_task_label_column_string]
         col['expandable'] = True
         col['resizable'] = True
         col['sorting'] = 'title'
@@ -460,14 +439,13 @@ class TreeviewFactory():
         treeview.set_expander_column('label')
         treeview.set_dnd_name('gtg/task-iter-str')
         # Background colors
-        treeview.set_bg_color(self.task_bg_color, 'bg_color')
+        treeview.set_bg_color(self.get_task_bg_color, 'bg_color')
         # Global treeview properties
         treeview.set_property("enable-tree-lines", False)
         treeview.set_rules_hint(False)
         treeview.set_multiple_selection(True)
         # Updating the unactive color (same for everyone)
-        color = treeview.get_style_context().get_color(
-            Gtk.StateFlags.INSENSITIVE)
+        color = treeview.get_style_context().get_color(Gtk.StateFlags.INSENSITIVE)
         # Convert color into #RRRGGGBBB
         self.unactive_color = color.to_color().to_string()
         return treeview
@@ -480,10 +458,9 @@ class TreeviewFactory():
         treeview.set_row_separator_func(self.is_tag_separator_filter)
         treeview.set_headers_visible(False)
         treeview.set_dnd_name('gtg/tag-iter-str')
-        treeview.set_dnd_external('gtg/task-iter-str', self.ontag_task_dnd)
+        treeview.set_dnd_external('gtg/task-iter-str', self.on_tag_task_dnd)
         # Updating the unactive color (same for everyone)
-        color = treeview.get_style_context().get_color(
-            Gtk.StateFlags.INSENSITIVE)
+        color = treeview.get_style_context().get_color(Gtk.StateFlags.INSENSITIVE)
         # Convert color into #RRRGGGBBB
         self.unactive_color = color.to_color().to_string()
 
