@@ -14,7 +14,7 @@
 # You should have received a copy of the GNU General Public License along with
 # this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from GTG.core.translations import _
+from gettext import gettext as _
 
 title = _("Foldable booklet (PDF)")
 description = _("""A template to create
