@@ -27,14 +27,15 @@ import errno
 import os
 import pickle
 import threading
+import logging
 
 from GTG.backends.backend_signals import BackendSignals
 from GTG.core.tag import ALLTASKS_TAG
 from GTG.core.dirs import SYNC_DATA_DIR
 from GTG.core.interruptible import _cancellation_point
 from GTG.core.keyring import Keyring
-from GTG.core.logger import log, log_debug_enabled
 
+log = logging.getLogger(__name__)
 PICKLE_BACKUP_NBR = 2
 
 
@@ -174,6 +175,7 @@ class GenericBackend():
     # the backend gtg internal name
     # (doesn't change in translations, *must be unique*)
     BACKEND_NAME = "name"
+    BACKEND_ICON = "icon"
     BACKEND_HUMAN_NAME = "human-friendly-name"  # The name shown to the user
     BACKEND_DESCRIPTION = "description"  # A short description of the backend
     BACKEND_AUTHORS = "authors"  # a list of strings
@@ -275,19 +277,21 @@ class GenericBackend():
             # if it's not specified, then this is the default backend
             # (for retro-compatibility with the GTG 0.2 series)
             parameters[self.KEY_DEFAULT_BACKEND] = True
+
         # default backends should get all the tasks
-        if parameters[self.KEY_DEFAULT_BACKEND] or \
-                (self.KEY_ATTACHED_TAGS not in parameters and
-                 self._general_description[self.BACKEND_TYPE] ==
-                    self.TYPE_READWRITE):
+        no_attached_tags = self.KEY_ATTACHED_TAGS not in parameters and \
+            self._general_description[self.BACKEND_TYPE] == self.TYPE_READWRITE
+
+        if parameters[self.KEY_DEFAULT_BACKEND] or no_attached_tags:
             parameters[self.KEY_ATTACHED_TAGS] = [ALLTASKS_TAG]
+
         self._parameters = parameters
         self._signal_manager = BackendSignals()
         self._is_initialized = False
         # if debugging mode is enabled, tasks should be saved as soon as
         # they're marked as modified. If in normal mode, we prefer speed over
         # easier debugging.
-        if log_debug_enabled():
+        if log.isEnabledFor(logging.DEBUG):
             self.timer_timestep = 5
         else:
             self.timer_timestep = 1
@@ -310,7 +314,7 @@ class GenericBackend():
             return [ALLTASKS_TAG]
         try:
             return self._parameters[self.KEY_ATTACHED_TAGS]
-        except:
+        except Exception:
             return []
 
     def set_attached_tags(self, tags):
@@ -350,6 +354,13 @@ class GenericBackend():
         """
         return cls._get_from_general_description(cls.BACKEND_NAME)
 
+    @classmethod
+    def get_icon(cls):
+        """
+        Returns the ison of the backend as it should be displayed in the UI
+        """
+        return cls._get_from_general_description(cls.BACKEND_ICON)
+
     @classmethod
     def get_description(cls):
         """Returns a description of the backend"""
@@ -399,7 +410,7 @@ class GenericBackend():
         elif param_type == cls.TYPE_PASSWORD:
             if param_value == -1:
                 return None
-            return Keyring().get_password(int(param_value))
+            return Keyring().get_password(param_value)
         elif param_type == cls.TYPE_LIST_OF_STRINGS:
             the_list = param_value.split(",")
             if not isinstance(the_list, list):
@@ -506,7 +517,7 @@ class GenericBackend():
         """
         try:
             return self.get_static_parameters()[param_name][self.PARAM_TYPE]
-        except:
+        except Exception:
             return None
 
     def register_datastore(self, datastore):
@@ -556,7 +567,7 @@ class GenericBackend():
 
         # saving
         with open(path, 'wb') as file:
-                pickle.dump(data, file)
+            pickle.dump(data, file)
 
     def _load_pickled_file(self, path, default_value=None):
         """
@@ -575,7 +586,7 @@ class GenericBackend():
             try:
                 return pickle.load(file)
             except Exception:
-                log.error("Pickle file for backend '%s' is damaged" %
+                log.error("Pickle file for backend '%s' is damaged",
                           self.get_name())
 
         # Loading file failed, trying backups
@@ -585,16 +596,16 @@ class GenericBackend():
                 with open(backup_file, 'rb') as file:
                     try:
                         data = pickle.load(file)
-                        log.info("Succesfully restored backup #%d for '%s'" %
-                                 (i, self.get_name()))
+                        log.info("Succesfully restored backup #%d for %r",
+                                 i, self.get_name())
                         return data
                     except Exception:
-                        log.error("Backup #%d for '%s' is damaged as well" %
-                                  (i, self.get_name()))
+                        log.error("Backup #%d for %r is damaged as well",
+                                  i, self.get_name())
 
         # Data could not be loaded, degrade to default data
-        log.error(f"There is no suitable backup for '{self.get_name()}', "
-                  "loading default data")
+        log.error("There is no suitable backup for %r, loading default data",
+                  self.get_name())
         return default_value
 
     def _gtg_task_is_syncable_per_attached_tags(self, task):
@@ -691,11 +702,11 @@ class GenericBackend():
             self.please_quit = True
             try:
                 self.to_set_timer.cancel()
-            except:
+            except Exception:
                 pass
             try:
                 self.to_set_timer.join()
-            except:
+            except Exception:
                 pass
         self.launch_setting_thread(bypass_quit_request=True)
         self.save_state()
