@@ -22,29 +22,33 @@ It's the window you see when you double-click on a Task
 The main text widget is a home-made TextView called TaskView (see taskview.py)
 The rest is the logic of the widget: date changing widgets, buttons, ...
 """
-import time
 import datetime
+import logging
 import os
+import time
+from gettext import gettext as _, ngettext
 
-from gi.repository import Gdk, Gtk, Pango, Gio
-
+from gi.repository import Gdk, Gtk, Pango
+from gi.repository.GObject import signal_handler_block
+from GTG.core.dates import Accuracy, Date
 from GTG.core.dirs import UI_DIR
 from GTG.core.plugins.api import PluginAPI
 from GTG.core.plugins.engine import PluginEngine
 from GTG.core.task import Task
-from GTG.core.translations import _, ngettext
 from GTG.gtk.editor import GnomeConfig
 from GTG.gtk.editor.calendar import GTGCalendar
+from GTG.gtk.editor.recurring_menu import RecurringMenu
 from GTG.gtk.editor.taskview import TaskView
 from GTG.gtk.tag_completion import tag_filter
-from GTG.core.dates import Date
-from GTG.core.logger import log
+from GTG.gtk.colors import rgb_to_hex
 """
 TODO (jakubbrindza): re-factor tag_filter into a separate module
 """
 
+log = logging.getLogger(__name__)
 
-class TaskEditor():
+
+class TaskEditor:
 
     EDITOR_UI_FILE = os.path.join(UI_DIR, "task_editor.ui")
 
@@ -90,6 +94,8 @@ class TaskEditor():
         self.due_entry = self.builder.get_object("duedate_entry")
         self.due_calendar = self.builder.get_object("calendar_due")
 
+        # Recurrence
+        self.recurring_menu = RecurringMenu(self.req, task.tid, self.builder)
 
         # Create our dictionary and connect it
         dic = {
@@ -98,9 +104,18 @@ class TaskEditor():
 
             "on_move": self.on_move,
 
+            "set_recurring_term_every_day": self.set_recurring_term_every_day,
+            "set_recurring_term_every_otherday": self.set_recurring_term_every_otherday,
+            "set_recurring_term_every_week": self.set_recurring_term_every_week,
+            "set_recurring_term_every_month": self.set_recurring_term_every_month,
+            "set_recurring_term_every_year": self.set_recurring_term_every_year,
+            "set_recurring_term_week_day": self.set_recurring_term_week_day,
+            "set_recurring_term_calender_month": self.set_recurring_term_month,
+            "set_recurring_term_calender_year": self.set_recurring_term_year,
+            "toggle_recurring_status": self.toggle_recurring_status,
+            "on_repeat_icon_toggled": self.on_repeat_icon_toggled,
+
             "show_popover_start": self.show_popover_start,
-            "startdate_selected": lambda c: self.on_date_selected(
-                c, GTGCalendar.DATE_KIND_START),
             "startingdate_changed": lambda w: self.date_changed(
                 w, GTGCalendar.DATE_KIND_START),
             "startdate_cleared": lambda w: self.on_date_cleared(
@@ -109,8 +124,6 @@ class TaskEditor():
                 w, e, GTGCalendar.DATE_KIND_START),
 
             "show_popover_due": self.show_popover_due,
-            "duedate_selected": lambda c: self.on_date_selected(
-                c, GTGCalendar.DATE_KIND_DUE),
             "duedate_changed": lambda w: self.date_changed(
                 w, GTGCalendar.DATE_KIND_DUE),
             "duedate_now_selected": lambda w: self.on_duedate_fuzzy(
@@ -127,16 +140,29 @@ class TaskEditor():
             "show_popover_closed": self.show_popover_closed,
             "closeddate_changed": lambda w: self.date_changed(
                 w, GTGCalendar.DATE_KIND_CLOSED),
-            "closeddate_selected": lambda c: self.on_date_selected(
-                c, GTGCalendar.DATE_KIND_CLOSED),
             "closeddate_focus_out": lambda w, e: self.date_focus_out(
                 w, e, GTGCalendar.DATE_KIND_CLOSED),
         }
 
-        self.builder.connect_signals(dic)
         self.window = self.builder.get_object("TaskEditor")
+        self.builder.connect_signals(dic)
         self.window.set_application(app)
 
+        if task.has_parent():
+            self.parent_button.set_label(_('Open Parent'))
+        else:
+            self.parent_button.set_label(_('Add Parent'))
+
+        # Connect signals for the calendar
+        self.start_handle = self.start_calendar.connect(
+            'day-selected', lambda c: self.on_date_selected(c, GTGCalendar.DATE_KIND_START))
+
+        self.due_handle = self.due_calendar.connect(
+            'day-selected', lambda c: self.on_date_selected(c, GTGCalendar.DATE_KIND_DUE))
+
+        self.closed_handle = self.closed_calendar.connect(
+            'day-selected', lambda c: self.on_date_selected(c, GTGCalendar.DATE_KIND_CLOSED))
+
         # Removing the Normal textview to replace it by our own
         # So don't try to change anything with glade, this is a home-made
         # widget
@@ -145,15 +171,27 @@ class TaskEditor():
         scrolled.remove(textview)
         self.textview = TaskView(self.req, self.clipboard)
         self.textview.show()
-        self.textview.set_subtask_callback(self.new_subtask)
-        self.textview.open_task_callback(self.app.open_task)
-        self.textview.set_left_margin(20)
-        self.textview.set_right_margin(20)
         scrolled.add(self.textview)
         conf_font_value = self.browser_config.get("font_name")
         if conf_font_value != "":
             self.textview.override_font(Pango.FontDescription(conf_font_value))
+
+        self.textview.browse_tag_cb = app.select_tag
+        self.textview.new_subtask_cb = self.new_subtask
+        self.textview.get_subtasks_cb = task.get_children
+        self.textview.delete_subtask_cb = self.remove_subtask
+        self.textview.rename_subtask_cb = self.rename_subtask
+        self.textview.open_subtask_cb = self.open_subtask
+        self.textview.save_cb = self.light_save
+        self.textview.add_tasktag_cb = task.tag_added
+        self.textview.remove_tasktag_cb = task.remove_tag
+        self.textview.refresh_cb = self.refresh_editor
+        self.textview.get_tagslist_cb = task.get_tags_name
+        self.textview.tid = task.tid
+
         # Voila! it's done
+        self.textview.connect('focus-in-event', self.on_textview_focus_in)
+        self.textview.connect('focus-out-event', self.on_textview_focus_out)
 
         """
         TODO(jakubbrindza): Once all the functionality in editor is back and
@@ -164,36 +202,41 @@ class TaskEditor():
 
         self.task = task
         tags = task.get_tags()
-        self.textview.subtasks_callback(task.get_children)
-        self.textview.removesubtask_callback(task.remove_child)
-        self.textview.set_get_tagslist_callback(task.get_tags_name)
-        self.textview.set_add_tag_callback(task.add_tag)
-        self.textview.set_remove_tag_callback(task.remove_tag)
-        self.textview.save_task_callback(self.light_save)
-
-        texte = self.task.get_text()
+        text = self.task.get_text()
         title = self.task.get_title()
-        # the first line is the title
-        self.textview.set_text(f"{title}\n")
-        # we insert the rest of the task
-        if texte:
-            self.textview.insert(f"{texte}")
+
+        # Insert text and tags as a non_undoable action, otherwise
+        # the user can CTRL+Z even this inserts.
+        self.textview.buffer.begin_not_undoable_action()
+        self.textview.buffer.set_text(f"{title}\n")
+
+        if text:
+            self.textview.insert(text)
+
+            # Insert any remaining tags
+            if tags:
+                tag_names = [t.get_name() for t in tags]
+                self.textview.insert_tags(tag_names)
         else:
             # If not text, we insert tags
             if tags:
-                for t in tags:
-                    self.textview.insert_text("%s, " % t.get_name())
-                self.textview.insert_text("\n")
-            # If we don't have text, we still need to insert subtasks if any
-            subtasks = task.get_children()
-            if subtasks:
-                self.textview.insert_subtasks(subtasks)
-        # We select the title if it's a new task
+                tag_names = [t.get_name() for t in tags]
+                self.textview.insert_tags(tag_names)
+                start = self.textview.buffer.get_end_iter()
+                self.textview.buffer.insert(start, '\n')
+
+        # Insert subtasks if they weren't inserted in the text
+        subtasks = task.get_children()
+        for sub in subtasks:
+            if sub not in self.textview.subtasks['tags']:
+                self.textview.insert_existing_subtask(sub)
+
         if thisisnew:
             self.textview.select_title()
         else:
             self.task.set_to_keep()
-        self.textview.modified(full=True)
+
+        self.textview.buffer.end_not_undoable_action()
         self.window.connect("destroy", self.destruction)
 
         # Connect search field to tags popup
@@ -210,7 +253,6 @@ class TaskEditor():
         self.pengine.onTaskLoad(self.plugin_api)
 
         # Putting the refresh callback at the end make the start a lot faster
-        self.textview.refresh_callback(self.refresh_editor)
         self.refresh_editor()
         self.textview.grab_focus()
 
@@ -218,48 +260,53 @@ class TaskEditor():
 
         self.window.insert_action_group('app', app)
         self.window.insert_action_group('win', app.browser)
-        self._set_actions()
 
         self.textview.set_editable(True)
+        self.window.set_transient_for(self.app.browser)
         self.window.show()
 
-    def _set_actions(self):
-        """Setup actions."""
-
-        action_entries = [
-            ('editor.close', self.close,
-             ('app.editor.close', ['Escape', '<ctrl>w'])),
-            ('editor.show_parent', self.on_parent_select, None),
-            ('editor.delete', self.delete_task, None),
-            ('editor.open_tags_popup', self.open_tags_popover, None),
-        ]
-
-        for action, callback, accel in action_entries:
-            simple_action = Gio.SimpleAction.new(action, None)
-            simple_action.connect('activate', callback)
-            simple_action.set_enabled(True)
-
-            self.app.add_action(simple_action)
-
-            if accel is not None:
-                self.app.set_accels_for_action(*accel)
-
     def show_popover_start(self, widget, event):
         """Open the start date calendar popup."""
 
+        start_date = (self.task.get_start_date() or Date.today()).date()
+
+        with signal_handler_block(self.start_calendar, self.start_handle):
+            self.start_calendar.select_day(start_date.day)
+            self.start_calendar.select_month(start_date.month - 1,
+                                             start_date.year)
+
         self.start_popover.popup()
 
     def show_popover_due(self, widget, popover):
         """Open the due date calendar popup."""
 
+        due_date = self.task.get_due_date()
+
+        if not due_date or due_date.is_fuzzy():
+            due_date = Date.today()
+
+        due_date = due_date.date()
+
+        with signal_handler_block(self.due_calendar, self.due_handle):
+            self.due_calendar.select_day(due_date.day)
+            self.due_calendar.select_month(due_date.month - 1,
+                                           due_date.year)
+
         self.due_popover.popup()
 
     def show_popover_closed(self, widget, popover):
         """Open the closed date calendar popup."""
 
+        closed_date = self.task.get_closed_date().date()
+
+        with signal_handler_block(self.closed_calendar, self.closed_handle):
+            self.closed_calendar.select_day(closed_date.day)
+            self.closed_calendar.select_month(closed_date.month - 1,
+                                              closed_date.year)
+
         self.closed_popover.popup()
 
-    def open_tags_popover(self, action, param):
+    def open_tags_popover(self):
         self.tag_store.clear()
 
         tags = self.req.get_tag_tree().get_all_nodes()
@@ -289,6 +336,61 @@ class TaskEditor():
         TODO(jakubbrindza): Add else case that will remove tag.
         """
 
+    def on_repeat_icon_toggled(self, widget):
+        """ Reset popup stack to the first page every time you open it """
+        if widget.get_active():
+            self.recurring_menu.reset_stack()
+
+    def toggle_recurring_status(self, widget):
+        self.recurring_menu.update_repeat_checkbox()
+        self.refresh_editor()
+
+    def set_recurring_term_every_day(self, widget):
+        self.recurring_menu.set_selected_term('day')
+        self.recurring_menu.update_term()
+        self.refresh_editor()
+
+    def set_recurring_term_every_otherday(self, widget):
+        self.recurring_menu.set_selected_term('other-day')
+        self.recurring_menu.update_term()
+        self.refresh_editor()
+
+    def set_recurring_term_every_week(self, widget):
+        self.recurring_menu.set_selected_term('week')
+        self.recurring_menu.update_term()
+        self.refresh_editor()
+
+    def set_recurring_term_every_month(self, widget):
+        self.recurring_menu.set_selected_term('month')
+        self.recurring_menu.update_term()
+        self.refresh_editor()
+
+    def set_recurring_term_every_year(self, widget):
+        self.recurring_menu.set_selected_term('year')
+        self.recurring_menu.update_term()
+        self.refresh_editor()
+
+    def set_recurring_term_week_day(self, widget):
+        self.recurring_menu.set_selected_term(widget.get_property("name"))
+        self.recurring_menu.update_term()
+        self.refresh_editor()
+
+    def set_recurring_term_month(self, widget):
+        self.recurring_menu.set_selected_term(str(widget.get_date()[2]))
+        self.recurring_menu.update_term()
+        self.refresh_editor()
+
+    def set_recurring_term_year(self, widget):
+        month = str(widget.get_date()[1] + 1)
+        day = str(widget.get_date()[2])
+        if len(month) < 2:
+            month = "0" + month
+        if len(day) < 2:
+            day = "0" + day
+        self.recurring_menu.set_selected_term(month + day)
+        self.recurring_menu.update_term()
+        self.refresh_editor()
+
     def search_function(self, model, column, key, iter, *search_data):
         """Callback when searching in the tags popup."""
 
@@ -299,25 +401,63 @@ class TaskEditor():
         # otherwise.
         return not model.get(iter, column)[0].startswith(key)
 
+    @staticmethod
+    def get_monitor_dimensions() -> Gdk.Rectangle:
+        """Get dimensions for the first monitor."""
+        return Gdk.Display.get_default().get_monitor(0).get_geometry()
+
     def init_dimensions(self):
         """ Restores position and size of task if possible """
-        position = self.config.get('position')
-        if position and len(position) == 2:
-            try:
-                self.window.move(int(position[0]), int(position[1]))
-            except ValueError:
-                log.warning(
-                    'Invalid position configuration for task %s: %s',
-                    self.task.get_id(), position)
 
+        position = self.config.get('position')
         size = self.config.get('size')
+        screen_size = self.get_monitor_dimensions()
+
         if size and len(size) == 2:
             try:
                 self.window.resize(int(size[0]), int(size[1]))
             except ValueError:
-                log.warning(
-                    'Invalid size configuration for task %s: %s',
-                    self.task.get_id(), size)
+                log.warning('Invalid size configuration for task %s: %s',
+                            self.task.get_id(), size)
+
+        can_move = True
+        if position and len(position) == 2:
+            try:
+                x = max(0, int(position[0]))
+                y = max(0, int(position[1]))
+                can_move = True
+            except ValueError:
+                can_move = False
+                log.warning('Invalid position configuration for task %s:%s',
+                            self.task.get_id(), position)
+        else:
+            gdk_window = self.window.get_window()
+            if gdk_window is None:
+                log.debug("Using default display to position editor window")
+                display = Gdk.Display.get_default()
+            else:
+                # TODO: AFAIK never happens because window is not realized at
+                #       this point, but maybe we should just in case the display
+                #       is actually different.
+                display = gdk_window.get_display()
+            seat = display.get_default_seat()
+            pointer = seat.get_pointer()
+            if pointer is None:
+                can_move = False
+                log.debug("Didn't receiver pointer info, can't move editor window")
+            else:
+                screen, x, y = pointer.get_position()
+                assert isinstance(x, int)
+                assert isinstance(y, int)
+
+        if can_move:
+            width, height = self.window.get_size()
+
+            # Clamp positions to current screen size
+            x = min(x, screen_size.width - width)
+            y = min(y, screen_size.height - height)
+
+            self.window.move(x, y)
 
     # Can be called at any time to reflect the status of the Task
     # Refresh should never interfere with the TaskView.
@@ -353,9 +493,13 @@ class TaskEditor():
             self.dismissbutton.show()
             self.undismissbutton.hide()
 
-        # Refreshing the the parent button
-        has_parents = len(self.task.get_parents()) > 0
-        self.parent_button.set_sensitive(has_parents)
+        # Refreshing the parent button
+        if self.task.has_parent():
+            # Translators: Button label to open the parent task
+            self.parent_button.set_label(_('Open Parent'))
+        else:
+            # Translators: Button label to add an new parent task
+            self.parent_button.set_label(_('Add Parent'))
 
         # Refreshing the status bar labels and date boxes
         if status in [Task.STA_DISMISSED, Task.STA_DONE]:
@@ -374,7 +518,7 @@ class TaskEditor():
             update_date = True
 
         if update_date:
-            self.start_entry.set_text(str(startdate))
+            self.start_entry.set_text(startdate.localized_str)
 
         # refreshing the due date field
         duedate = self.task.get_due_date()
@@ -385,13 +529,13 @@ class TaskEditor():
             update_date = True
 
         if update_date:
-            self.due_entry.set_text(str(duedate))
+            self.due_entry.set_text(duedate.localized_str)
 
         # refreshing the closed date field
         closeddate = self.task.get_closed_date()
         prevcldate = Date.parse(self.closed_entry.get_text())
         if closeddate != prevcldate:
-            self.closed_entry.set_text(str(closeddate))
+            self.closed_entry.set_text(closeddate.localized_str)
 
         # refreshing the day left label
         """
@@ -444,7 +588,7 @@ class TaskEditor():
         style_context = self.window.get_style_context()
         color = style_context.get_color(Gtk.StateFlags.INSENSITIVE).to_color()
         self.dayleft_label.set_markup(
-            f"<span color='{color.to_string()}'>{txt}</span>")
+            f"<span color='{rgb_to_hex(color)}'>{txt}</span>")
 
     def reload_editor(self):
         task = self.task
@@ -500,7 +644,6 @@ class TaskEditor():
 
             self.refresh_editor()
 
-
     def calendar_to_datetime(self, calendar):
         """
         Gtk.Calendar uses a 0-based convention for counting months.
@@ -516,7 +659,7 @@ class TaskEditor():
         """ Callback when a fuzzy date is selected through the popup. """
 
         self.task.set_due_date(date)
-        self.due_entry.set_text(str(date))
+        self.due_entry.set_text(date.localized_str)
 
     def on_date_cleared(self, widget, kind):
         """ Callback when a date is cleared through the popups. """
@@ -536,17 +679,15 @@ class TaskEditor():
 
         if kind == GTGCalendar.DATE_KIND_START:
             self.task.set_start_date(Date(date))
-            self.start_entry.set_text(str(Date(date)))
+            self.start_entry.set_text(Date(date).localized_str)
 
         elif kind == GTGCalendar.DATE_KIND_DUE:
             self.task.set_due_date(Date(date))
-            self.due_entry.set_text(str(Date(date)))
+            self.due_entry.set_text(Date(date).localized_str)
 
         elif kind == GTGCalendar.DATE_KIND_CLOSED:
             self.task.set_closed_date(Date(date))
-            self.closed_entry.set_text(str(Date(date)))
-
-
+            self.closed_entry.set_text(Date(date).localized_str)
 
     def on_date_changed(self, calendar):
         date, date_kind = calendar.get_selected_date()
@@ -592,12 +733,16 @@ class TaskEditor():
             self.close_all_subtasks()
             self.close(None)
 
-    def delete_task(self, action, param):
-        # this triggers the closing of the window in the view manager
-        if self.task.is_new():
-            self.app.close_task(self.task.get_id(), self.window)
-        else:
-            self.app.ask_delete_tasks([self.task.get_id()], self.window)
+    def reopen(self):
+        self.task.set_status(Task.STA_ACTIVE)
+        self.refresh_editor()
+
+    def open_subtask(self, tid):
+        """Open subtask (closing parent task)."""
+
+        task = self.req.get_task(tid)
+        self.app.open_task(tid)
+        self.app.close_task(task.parents[0])
 
     # Take the title as argument and return the subtask ID
     def new_subtask(self, title=None, tid=None):
@@ -609,8 +754,23 @@ class TaskEditor():
             tid = subt.get_id()
         return tid
 
+    def remove_subtask(self, tid):
+        """Remove a subtask of this task."""
+
+        self.task.remove_child(tid)
+
+    def rename_subtask(self, tid, new_title):
+        """Rename a subtask of this task."""
+
+        try:
+            self.req.get_task(tid).set_title(new_title)
+        except AttributeError:
+            # There's no task at that tid
+            pass
+
+
     def insert_subtask(self, action=None, param=None):
-        self.textview.insert_newtask()
+        self.textview.insert_new_subtask()
         self.textview.grab_focus()
 
     def inserttag_clicked(self, widget):
@@ -620,11 +780,30 @@ class TaskEditor():
         else:
             self.textview.insert_text(" @", itera)
 
-    def on_parent_select(self, action, param):
+    def open_parent(self):
+        """
+        Open (or create) the parent task,
+        then close the child to avoid various window management issues
+        and to prevent visible content divergence when the child title changes.
+        """
         parents = self.task.get_parents()
+        self.save()
 
-        if len(parents) == 1:
+        if not parents:
+            tags = [t.get_name() for t in self.task.get_tags()]
+            parent = self.req.new_task(tags=tags, newtask=True)
+            parent_id = parent.get_id()
+
+            self.task.set_parent(parent_id)
+            self.app.open_task(parent_id)
+            # Prevent WM issues and risks of conflicting content changes:
+            self.close()
+
+        elif len(parents) == 1:
             self.app.open_task(parents[0])
+            # Prevent WM issues and risks of conflicting content changes:
+            self.close()
+
         elif len(parents) > 1:
             self.show_multiple_parent_popover(parents)
 
@@ -643,11 +822,13 @@ class TaskEditor():
         self.parent_popover.set_transitions_enabled(True)
         self.parent_popover.show_all()
 
-    # On click handler for open_parent_button's menu items
+    # On click handler for open_parent menu items in the case of multiple parents
     def on_parent_item_clicked(self, widget, parent_id):
         self.app.open_task(parent_id)
         if self.parent_popover.get_visible():
             self.parent_popover.hide()
+        # Prevent WM issues and risks of conflicting content changes:
+        self.close()
 
     def save(self):
         self.task.set_title(self.textview.get_title())
@@ -671,8 +852,11 @@ class TaskEditor():
         if tosave:
             self.save()
 
-    # This will bring the Task Editor to front
     def present(self):
+        # This tries to bring the Task Editor to the front.
+        # If TaskEditor is a "utility" window type, this doesn't work on X11,
+        # it only works on GNOME's Wayland session, unless the child is closed.
+        # This is partly why we use self.close() in various places elsewhere.
         self.window.present()
 
     def get_position(self):
@@ -680,8 +864,15 @@ class TaskEditor():
 
     def on_move(self, widget, event):
         """ Save position and size of window """
-        self.config.set('position', self.window.get_position())
-        self.config.set('size', self.window.get_size())
+
+        self.config.set('position', list(self.window.get_position()))
+        self.config.set('size', list(self.window.get_size()))
+
+    def on_textview_focus_in(self, widget, event):
+        self.app.browser.toggle_delete_accel(False)
+
+    def on_textview_focus_out(self, widget, event):
+        self.app.browser.toggle_delete_accel(True)
 
     # We define dummy variable for when close is called from a callback
     def close(self, action=None, param=None):
@@ -691,24 +882,25 @@ class TaskEditor():
             self.window.destroy()
             self.window = None
 
-    # The destroy signal is linked to the "close" button. So if we call
-    # destroy in the close function, this will cause the close to be called
-    # twice
-    # To solve that, close will just call "destroy" and the destroy signal
-    # Will be linked to this destruction method that will save the task
-    def destruction(self, a=None):
+    def destruction(self, _=None):
+        """Callback when destroying the window."""
+
         # Save should be also called when buffer is modified
         self.pengine.onTaskClose(self.plugin_api)
         self.pengine.remove_api(self.plugin_api)
+
         tid = self.task.get_id()
+
         if self.task.is_new():
             self.req.delete_task(tid)
         else:
             self.save()
-            for i in self.task.get_subtasks():
-                if i:
-                    i.set_to_keep()
-        self.app.close_task(tid)
+            [sub.set_to_keep() for sub in self.task.get_subtasks() if sub]
+
+        try:
+            del self.app.open_tasks[tid]
+        except KeyError:
+            log.debug('Task %s was already removed from the open list', tid)
 
     def get_builder(self):
         return self.builder
