@@ -1,4 +1,4 @@
-# -*- coding: utf-8 -*-
+
 # Copyright (c) 2009 - Paulo Cabido <paulo.cabido@gmail.com>
 #
 # This program is free software: you can redistribute it and/or modify it under
@@ -14,8 +14,5 @@
 # You should have received a copy of the GNU General Public License along with
 # this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from GTG.plugins.hamster.hamster import hamsterPlugin
-
-
-#suppress pyflakes warning (given by make lint)
-if False == True: hamsterPlugin()
+from GTG.plugins.hamster.hamster import HamsterPlugin
+assert HamsterPlugin
